@@ -1,90 +1,61 @@
 from __future__ import annotations
 
 import operator
-from functools import partial
 from functools import reduce
 from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Container
 from typing import Iterable
-from typing import Literal
 from typing import Sequence
+from typing import cast
 
 import dask.dataframe as dd
 import pandas as pd
 
+from narwhals._compliant import CompliantThen
+from narwhals._compliant import CompliantWhen
+from narwhals._compliant import LazyNamespace
+from narwhals._compliant.namespace import DepthTrackingNamespace
 from narwhals._dask.dataframe import DaskLazyFrame
 from narwhals._dask.expr import DaskExpr
 from narwhals._dask.selectors import DaskSelectorNamespace
 from narwhals._dask.utils import align_series_full_broadcast
-from narwhals._dask.utils import name_preserving_div
-from narwhals._dask.utils import name_preserving_sum
 from narwhals._dask.utils import narwhals_to_native_dtype
 from narwhals._dask.utils import validate_comparand
 from narwhals._expression_parsing import combine_alias_output_names
 from narwhals._expression_parsing import combine_evaluate_output_names
-from narwhals.typing import CompliantNamespace
 from narwhals.utils import Implementation
-from narwhals.utils import exclude_column_names
-from narwhals.utils import get_column_names
-from narwhals.utils import passthrough_column_names
 
 if TYPE_CHECKING:
-    from typing_extensions import Self
+    import dask.dataframe.dask_expr as dx
 
     from narwhals.dtypes import DType
+    from narwhals.typing import ConcatMethod
+    from narwhals.typing import NonNestedLiteral
     from narwhals.utils import Version
 
-    try:
-        import dask.dataframe.dask_expr as dx
-    except ModuleNotFoundError:  # pragma: no cover
-        import dask_expr as dx
 
-
-class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright: ignore[reportInvalidTypeArguments] (#2044)
+class DaskNamespace(
+    LazyNamespace[DaskLazyFrame, DaskExpr, dd.DataFrame],
+    DepthTrackingNamespace[DaskLazyFrame, DaskExpr],
+):
     _implementation: Implementation = Implementation.DASK
 
     @property
-    def selectors(self: Self) -> DaskSelectorNamespace:
-        return DaskSelectorNamespace(self)
+    def selectors(self) -> DaskSelectorNamespace:
+        return DaskSelectorNamespace.from_namespace(self)
 
-    def __init__(
-        self: Self, *, backend_version: tuple[int, ...], version: Version
-    ) -> None:
+    @property
+    def _expr(self) -> type[DaskExpr]:
+        return DaskExpr
+
+    @property
+    def _lazyframe(self) -> type[DaskLazyFrame]:
+        return DaskLazyFrame
+
+    def __init__(self, *, backend_version: tuple[int, ...], version: Version) -> None:
         self._backend_version = backend_version
         self._version = version
 
-    def all(self: Self) -> DaskExpr:
-        return DaskExpr.from_column_names(
-            get_column_names,
-            function_name="all",
-            backend_version=self._backend_version,
-            version=self._version,
-        )
-
-    def col(self: Self, *column_names: str) -> DaskExpr:
-        return DaskExpr.from_column_names(
-            passthrough_column_names(column_names),
-            function_name="col",
-            backend_version=self._backend_version,
-            version=self._version,
-        )
-
-    def exclude(self: Self, excluded_names: Container[str]) -> DaskExpr:
-        return DaskExpr.from_column_names(
-            partial(exclude_column_names, names=excluded_names),
-            function_name="exclude",
-            backend_version=self._backend_version,
-            version=self._version,
-        )
-
-    def nth(self: Self, *column_indices: int) -> DaskExpr:
-        return DaskExpr.from_column_indices(
-            *column_indices, backend_version=self._backend_version, version=self._version
-        )
-
-    def lit(self: Self, value: Any, dtype: DType | None) -> DaskExpr:
+    def lit(self, value: NonNestedLiteral, dtype: DType | type[DType] | None) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             if dtype is not None:
                 native_dtype = narwhals_to_native_dtype(dtype, self._version)
@@ -95,7 +66,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             dask_series = dd.from_pandas(native_pd_series, npartitions=npartitions)
             return [dask_series[0].to_series()]
 
-        return DaskExpr(
+        return self._expr(
             func,
             depth=0,
             function_name="lit",
@@ -105,19 +76,12 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def len(self: Self) -> DaskExpr:
+    def len(self) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
-            if not df.columns:
-                return [
-                    dd.from_pandas(
-                        pd.Series([0], name="len"),
-                        npartitions=df._native_frame.npartitions,
-                    )
-                ]
+            # We don't allow dataframes with 0 columns, so `[0]` is safe.
             return [df._native_frame[df.columns[0]].size.to_series()]
 
-        # coverage bug? this is definitely hit
-        return DaskExpr(  # pragma: no cover
+        return self._expr(
             func,
             depth=0,
             function_name="len",
@@ -127,14 +91,14 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def all_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def all_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             series = align_series_full_broadcast(
                 df, *(s for _expr in exprs for s in _expr(df))
             )
             return [reduce(operator.and_, series)]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="all_horizontal",
@@ -144,14 +108,14 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def any_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def any_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             series = align_series_full_broadcast(
                 df, *(s for _expr in exprs for s in _expr(df))
             )
             return [reduce(operator.or_, series)]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="any_horizontal",
@@ -161,14 +125,14 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def sum_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def sum_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             series = align_series_full_broadcast(
                 df, *(s for _expr in exprs for s in _expr(df))
             )
             return [dd.concat(series, axis=1).sum(axis=1)]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="sum_horizontal",
@@ -179,10 +143,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
         )
 
     def concat(
-        self: Self,
-        items: Iterable[DaskLazyFrame],
-        *,
-        how: Literal["horizontal", "vertical", "diagonal"],
+        self, items: Iterable[DaskLazyFrame], *, how: ConcatMethod
     ) -> DaskLazyFrame:
         if not items:
             msg = "No items to concatenate"  # pragma: no cover
@@ -206,24 +167,6 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
                 backend_version=self._backend_version,
                 version=self._version,
             )
-        if how == "horizontal":
-            all_column_names: list[str] = [
-                column for frame in dfs for column in frame.columns
-            ]
-            if len(all_column_names) != len(set(all_column_names)):  # pragma: no cover
-                duplicates = [
-                    i for i in all_column_names if all_column_names.count(i) > 1
-                ]
-                msg = (
-                    f"Columns with name(s): {', '.join(duplicates)} "
-                    "have more than one occurrence"
-                )
-                raise AssertionError(msg)
-            return DaskLazyFrame(
-                dd.concat(dfs, axis=1, join="outer"),
-                backend_version=self._backend_version,
-                version=self._version,
-            )
         if how == "diagonal":
             return DaskLazyFrame(
                 dd.concat(dfs, axis=0, join="outer"),
@@ -233,21 +176,18 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
 
         raise NotImplementedError
 
-    def mean_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def mean_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             expr_results = [s for _expr in exprs for s in _expr(df)]
             series = align_series_full_broadcast(df, *(s.fillna(0) for s in expr_results))
             non_na = align_series_full_broadcast(
                 df, *(1 - s.isna() for s in expr_results)
             )
-            return [
-                name_preserving_div(
-                    reduce(name_preserving_sum, series),
-                    reduce(name_preserving_sum, non_na),
-                )
-            ]
+            num = reduce(lambda x, y: x + y, series)  # pyright: ignore[reportOperatorIssue]
+            den = reduce(lambda x, y: x + y, non_na)  # pyright: ignore[reportOperatorIssue]
+            return [cast("dx.Series", num / den)]  # pyright: ignore[reportOperatorIssue]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="mean_horizontal",
@@ -257,7 +197,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def min_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def min_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             series = align_series_full_broadcast(
                 df, *(s for _expr in exprs for s in _expr(df))
@@ -265,7 +205,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
 
             return [dd.concat(series, axis=1).min(axis=1)]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="min_horizontal",
@@ -275,7 +215,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def max_horizontal(self: Self, *exprs: DaskExpr) -> DaskExpr:
+    def max_horizontal(self, *exprs: DaskExpr) -> DaskExpr:
         def func(df: DaskLazyFrame) -> list[dx.Series]:
             series = align_series_full_broadcast(
                 df, *(s for _expr in exprs for s in _expr(df))
@@ -283,7 +223,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
 
             return [dd.concat(series, axis=1).max(axis=1)]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="max_horizontal",
@@ -293,11 +233,11 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
             version=self._version,
         )
 
-    def when(self: Self, predicate: DaskExpr) -> DaskWhen:
-        return DaskWhen(predicate, self._backend_version, version=self._version)
+    def when(self, predicate: DaskExpr) -> DaskWhen:
+        return DaskWhen.from_expr(predicate, context=self)
 
     def concat_str(
-        self: Self,
+        self,
         *exprs: DaskExpr,
         separator: str,
         ignore_nulls: bool,
@@ -331,7 +271,7 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
 
             return [result]
 
-        return DaskExpr(
+        return self._expr(
             call=func,
             depth=max(x._depth for x in exprs) + 1,
             function_name="concat_str",
@@ -344,23 +284,12 @@ class DaskNamespace(CompliantNamespace[DaskLazyFrame, "dx.Series"]):  # pyright:
         )
 
 
-class DaskWhen:
-    def __init__(
-        self: Self,
-        condition: DaskExpr,
-        backend_version: tuple[int, ...],
-        then_value: Any = None,
-        otherwise_value: Any = None,
-        *,
-        version: Version,
-    ) -> None:
-        self._backend_version = backend_version
-        self._condition: DaskExpr = condition
-        self._then_value: DaskExpr | Any = then_value
-        self._otherwise_value: DaskExpr | Any = otherwise_value
-        self._version = version
+class DaskWhen(CompliantWhen[DaskLazyFrame, "dx.Series", DaskExpr]):
+    @property
+    def _then(self) -> type[DaskThen]:
+        return DaskThen
 
-    def __call__(self: Self, df: DaskLazyFrame) -> Sequence[dx.Series]:
+    def __call__(self, df: DaskLazyFrame) -> Sequence[dx.Series]:
         condition = self._condition(df)[0]
 
         if isinstance(self._then_value, DaskExpr):
@@ -376,50 +305,10 @@ class DaskWhen:
         if isinstance(self._otherwise_value, DaskExpr):
             otherwise_value = self._otherwise_value(df)[0]
         else:
-            return [then_series.where(condition, self._otherwise_value)]
+            return [then_series.where(condition, self._otherwise_value)]  # pyright: ignore[reportArgumentType]
         (otherwise_series,) = align_series_full_broadcast(df, otherwise_value)
         validate_comparand(condition, otherwise_series)
         return [then_series.where(condition, otherwise_series)]  # pyright: ignore[reportArgumentType]
 
-    def then(self: Self, value: DaskExpr | Any) -> DaskThen:
-        self._then_value = value
 
-        return DaskThen(
-            self,
-            depth=0,
-            function_name="whenthen",
-            evaluate_output_names=getattr(
-                value, "_evaluate_output_names", lambda _df: ["literal"]
-            ),
-            alias_output_names=getattr(value, "_alias_output_names", None),
-            backend_version=self._backend_version,
-            version=self._version,
-        )
-
-
-class DaskThen(DaskExpr):
-    def __init__(
-        self: Self,
-        call: DaskWhen,
-        *,
-        depth: int,
-        function_name: str,
-        evaluate_output_names: Callable[[DaskLazyFrame], Sequence[str]],
-        alias_output_names: Callable[[Sequence[str]], Sequence[str]] | None,
-        backend_version: tuple[int, ...],
-        version: Version,
-        call_kwargs: dict[str, Any] | None = None,
-    ) -> None:
-        self._backend_version = backend_version
-        self._version = version
-        self._call: DaskWhen = call
-        self._depth = depth
-        self._function_name = function_name
-        self._evaluate_output_names = evaluate_output_names
-        self._alias_output_names = alias_output_names
-        self._call_kwargs = call_kwargs or {}
-
-    def otherwise(self: Self, value: DaskExpr | Any) -> DaskExpr:
-        self._call._otherwise_value = value
-        self._function_name = "whenotherwise"
-        return self
+class DaskThen(CompliantThen[DaskLazyFrame, "dx.Series", DaskExpr], DaskExpr): ...
